% Simulation of a Vacuum-cleaner model with an extended Kalman filter

N = 200; % Number of time-steps to simulate

x = [
    0.0
    0.0
    0.0
    0.9
    1.1
    ];

t = 0;

xs = zeros(length(x), N);
xs(:, 1) = x;

xhat = [
    1.0
    1.0
    1.0
    1.0
    1.0
    ];


dt = 0.05; % Also used in f

xhats = zeros(length(x), N);
xhats(:, 1) = xhat;

V = 0.1*eye(length(g(x))); % Noise in measurements
invV = inv(V);

W = sqrt(dt).*diag([0.01, 0.01, 0.01, 0.0001, 0.0001]); % Noise in state

P = diag([1.0, 1.0, 1.0, 0.02, 0.02]); % Initial uncertainty in xhat

Pdiags = zeros(length(x), N);
Pdiags(:, 1) = diag(P);

for k=1:N-1
    % Update the simulation
    
    x = f(t, x);
    x = x + sqrtm(W)*randn(size(x));
    
    xs(:, k+1) = x;

    % Get a measurement
    y = g(x);
    y = y + sqrtm(V)*randn(size(y));

    % -------------------------------------------------

    % Prediction step
    A = dfdx(t, xhat); % Compute linearization at old xhat
    P = A*P*A' + W;

    xbar = f(t, xhat);

    C = dgdx(xbar); % Compute linearization at xbar

    % Innovation update
    P = inv( inv(P) + C' * invV * C);
    xhat = xbar + P * C' * invV * (y - g(xbar));
    xhats(:, k+1) = xhat;
    Pdiags(:, k+1) = diag(P);

    % ---------------------------------------------------

    t = t + dt;
end

names = {'x', 'y', '\beta', 'r_1', 'r_2'};

figure(1)
subplot(2,2,1)
plot(xs')
title('Actual state')
legend(names)

subplot(2,2,2)
plot(xhats')
title('Estimated state')
legend(names)

subplot(2,2,3)
plot(xhats' - xs')
title('Estimation error')
legend(names)

subplot(2,2,4)
plot(sqrt(Pdiags)')
title('sqrt(diagonal elements of P)')
legend(names)
